import 'dart:ui';

import 'package:flutter/services.dart';

enum ActivationMethod { NONE, SHAKE, SCREENSHOT }

String getActivationMethodValue(ActivationMethod activationMethod) {
  switch (activationMethod) {
    case ActivationMethod.NONE:
      return "NONE";
    case ActivationMethod.SHAKE:
      return "SHAKE";
    case ActivationMethod.SCREENSHOT:
      return "SCREENSHOT";
    default:
      return "NONE";
  }
}

enum BugPriority { LOW, MEDIUM, HIGH }

String getBugPriorityValue(BugPriority bugPriority) {
  switch (bugPriority) {
    case BugPriority.LOW:
      return "LOW";
    case BugPriority.MEDIUM:
      return "MEDIUM";
    case BugPriority.HIGH:
      return "HIGH";
    default:
      return "MEDIUM";
  }
}

typedef void BugBattleCustomActionCallback(String name);

class BugBattle {
  static const MethodChannel _channel = const MethodChannel('bugbattle_sdk');

  /// Sets a callback for receiving method calls on this channel.
  ///
  /// The given callback will replace the currently registered callback for this
  /// channel, if any. To remove the handler, pass null as the
  /// `handler` argument.
  static void addCustomActionCallback(
      Future<dynamic> Function(MethodCall call)? handler) {
    _channel.setMethodCallHandler(handler);
  }

  /// initializes bugbattle with [apiKey] and sets the [activationMethod].
  static void initWithToken(String apiKey, ActivationMethod activationMethod) {
    String activationMethodValue = getActivationMethodValue(activationMethod);
    _channel.invokeMethod(
      'initWithToken',
      {'apiKey': apiKey, 'activationMethod': activationMethodValue},
    );
  }

  /// initializes bugbattle with [apiKey] and sets the array of [activationMethods].
  static void initializeMany(
      String apiKey, List<ActivationMethod> activationMethods) {
    List<String> activationMethodsValues = [];
    for (int i = 0; i < activationMethods.length; i++) {
      String activationMethodString =
          getActivationMethodValue(activationMethods[i]);
      activationMethodsValues.add(activationMethodString);
    }

    _channel.invokeMethod('initializeMany',
        {'apiKey': apiKey, "activationMethods": activationMethodsValues});
  }

  /// auto configure bugbattle with your [apiKey].
  static void autoConfigure(String apiKey) {
    _channel.invokeMethod('autoConfigure', {"apiKey": apiKey});
  }

  /// starts bug reporting.
  static startBugReporting() {
    _channel.invokeMethod('startBugReporting');
  }

  /// enables replays.
  static void enableReplays() {
    _channel.invokeMethod('enableReplays');
  }

  /// sets a customer email.
  static void setCustomerEmail(String email) {
    _channel.invokeMethod('setCustomerEmail', {"email": email});
  }

  /// sends a silent bug report.
  static void sendSilentBugReport(
    String email,
    String description,
    BugPriority priority,
  ) {
    String bugPriorityVal = getBugPriorityValue(priority);

    _channel.invokeMethod('sendSilentBugReport', {
      "email": email,
      "description": description,
      "priority": bugPriorityVal
    });
  }

  /// attachs custom data.
  static void attachCustomData(Map<String, dynamic> customData) async {
    _channel.invokeMethod('attachCustomData', {"customData": customData});
  }

  /// sets custom data [data] with [key].
  static void setCustomData(String key, String value) {
    _channel.invokeMethod('setCustomData', {"key": key, "value": value});
  }

  /// remove custom data with [key].
  static void removeCustomData(String key) {
    _channel.invokeMethod('removeCustomData', {"key": key});
  }

  /// clears all custom data.
  static void clearCustomData() {
    _channel.invokeMethod('clearCustomData');
  }

  /// [enable] enables the pricacy policy.
  static void enablePrivacyPolicy(bool enable) {
    _channel.invokeMethod('enablePrivacyPolicy', {"enable": enable});
  }

  /// Sets the privacy url url to [url].
  static void setPrivacyPolicyUrl(String url) {
    _channel.invokeMethod('setPrivacyPolicyUrl', {"url": url});
  }

  /// Sets the logoUrl url to [logoUrl].
  static void enablePoweredByBugbattle(bool enabled) {
    _channel.invokeMethod('enablePoweredByBugbattle', {"enabled": enabled});
  }

  /// Sets the logoUrl url to [logoUrl].
  static void setLogoUrl(String logoUrl) {
    _channel.invokeMethod('setLogoUrl', {"logoUrl": logoUrl});
  }

  /// Sets the primary color.
  static void setColor(Color color) {
    final String hexString = _colorToHex(color);
    _channel.invokeMethod('setColor', {"color": hexString});
  }

  /// Sets the api url to [url].
  static void setApiUrl(String url) {
    _channel.invokeMethod('setApiUrl', {"url": url});
  }

  static String _colorToHex(Color color) {
    return '#${(color.value & 0xFFFFFF).toRadixString(16).padLeft(6, '0').toUpperCase()}';
  }

  static void setLanguage(String language) {
    _channel.invokeMethod('setLanguage', {"language": language});
  }
}
